$(function() {

	var XYZ2CIELab = function(XYZ) {
		var CIELab = {};
		// Observer = 2°, Illuminant = D65
		var X = XYZ.x / 95.047;
		var Y = XYZ.y / 100.000;
		var Z = XYZ.z / 108.883;

		if (X > 0.008856) {
			X = Math.pow(X, 0.333333333);
		}
		else {
			X = 7.787 * X + 0.137931034;
		}

		if (Y > 0.008856) {
			Y = Math.pow(Y, 0.333333333);
		}
		else {
			Y = 7.787 * Y + 0.137931034;
		}

		if (Z > 0.008856) {
			Z = Math.pow(Z, 0.333333333);
		}
		else {
			Z = 7.787 * Z + 0.137931034;
		}

		CIELab.l = (116 * Y) - 16;
		CIELab.a = 500 * (X - Y);
		CIELab.b = 200 * (Y - Z);

		return CIELab;
	};

	var rgb2XYZ = function(rgb) {
		var XYZ = {};

		var r = (rgb.r / 255);
		var g = (rgb.g / 255);
		var b = (rgb.b / 255);

		if (r > 0.04045) {
			r = Math.pow(((r + 0.055) / 1.055), 2.4);
		}
		else {
			r = r / 12.92;
		}

		if (g > 0.04045) {
			g = Math.pow(((g + 0.055) / 1.055), 2.4);
		}
		else {
			g = g / 12.92;
		}

		if (b > 0.04045) {
			b = Math.pow(((b + 0.055) / 1.055), 2.4);
		}
		else {
			b = b / 12.92;
		}

		r = r * 100;
		g = g * 100;
		b = b * 100;

		// Observer = 2°, Illuminant = D65
		XYZ.x = r * 0.4124 + g * 0.3576 + b * 0.1805;
		XYZ.y = r * 0.2126 + g * 0.7152 + b * 0.0722;
		XYZ.z = r * 0.0193 + g * 0.1192 + b * 0.9505;

		return XYZ;
	};

	function cielch (CIELab) {
		CIELab = XYZ2CIELab(rgb2XYZ(CIELab));
		var CIELCH = {};

		CIELCH.l = CIELab.l;
		CIELCH.c = Math.sqrt(Math.pow(CIELab.a, 2) + Math.pow(CIELab.b, 2));

		CIELCH.h = Math.atan2(CIELab.b, CIELab.a);  //Quadrant by signs

		if (CIELCH.h > 0) {
			CIELCH.h = (CIELCH.h / Math.PI) * 180;
		}
		else {
			CIELCH.h = 360 - (Math.abs(CIELCH.h) / Math.PI) * 180;
		}

		return CIELCH;
	};

	$('.color').each(function() {
		var el = this;
		var rgb_arr = $(this).css("backgroundColor").match(/\d+/g);
		var rgb ={};
		rgb.r = rgb_arr[0];
		rgb.g = rgb_arr[1];
		rgb.b = rgb_arr[2];
		var c = cielch(rgb);
		if (c.l < 60) {
			$(el).css("color", "white");
		}
		else {
			$(el).css("color", "black");
		}
	});



	var cssShowColor = function(el,rgb,hex) {
		var c = cielch(rgb);
		// 	$(el).css("backgroundColor","#"+hex);
		$(el).closest(".input-group").find(".color").css("backgroundColor","#"+hex);

		if (c.l < 60) {
			$(el).closest(".input-group").find(".color").css("color","white");
		}
		else {
			$(el).closest(".input-group").find(".color").css("color","black");
		}
	};

	$('.colorcode').colpick({
		colorScheme:'dark',
		onChange:function(hsb,hex,rgb,el,bySetColor) {
			cssShowColor(el,rgb,hex);
			$(el).val('#'+hex);
			//       $(el).closest(".input-group").find(".color").val("#" + hex);
		},

		onSubmit:function(hsb,hex,rgb,el,bySetColor) {
			cssShowColor(el,rgb,hex);
			//     $(el).closest(".input-group").find("input").val("#" + hex);

			$(el).val('#'+hex);
			$(el).colpickHide();
		}

	});


	$(".input-group-addon.color").on("click", function() {
		$(this).closest(".input-group").find(".colorcode").trigger("click");
	});
});

(function () {
	'use strict';

	var cssUploadFormularAdminClass = function (options) {
		this.init(options);
	};

	cssUploadFormularAdminClass.VERSION = '2.0.0';
	cssUploadFormularAdminClass.DEFAULTS = {
		showDebug : true,
		templateSelectors:  {
			'nova': {
				'default': {
					selector_position: "prepend",
					selector: "#content"
				},
			},
			'snackys': {
				'default': {
					selector_position: "prepend",
					selector: "#content"
				},
			},
			'admorris': {
				'default': {
					selector_position: "prepend",
					selector: "#content"
				},
			},
			'salepix': {
				'default': {
					selector_position: "prepend",
					selector: "#content"
				},
			},
			'easytemplate': {
				'default': {
					selector_position: "prepend",
					selector: "#content"
				},
			},
		},
	};

	cssUploadFormularAdminClass.prototype = {

		constructor: cssUploadFormularAdminClass,

		init: function (options) {
			this.options = $.extend({}, cssUploadFormularAdminClass.DEFAULTS, options);
			var that = this;
		},

		register: function(options) {
			var that=this;
			this.debug("ich starte cssUploadFormularAdminClass.VERSION " +  this.__proto__.constructor.VERSION);
			that.initTemplateSelectors();
			that.docu();
			that.codemirror();
		},

		debug: function(s) {
			if(this.options.showDebug) {
				console.log(s);
			}
		},

		_set: function(key,value) {
			this.options.key = value;
		},

		docu: function() {
			var that = this;
			$(document).on("click", "[data-section]", function () {
				var pluginID = $(this).closest("form").data('plugin');
				var data = "io=" + pluginID +  "&jtl_token=" + $(this).closest("form").data("session") + "&action=docu&section=" + $(this).data('section');
				var url = $(this).closest("form").data("io");
				that.post_data(url, data);
			});
		},


		initTemplateSelectors: function() {
			var that = this;
			$(document).on("click", "[data-template]", function() {
				var tSelectors = that.options.templateSelectors[$(this).data('template')];
				console.log(tSelectors);
				Object.entries(tSelectors).forEach(entry => {
					const [key, value] = entry;
					Object.entries(value).forEach(entry => {
						const [key, value] = entry;
						console.log(key,value);
						$("[name='" + key + "']").val(value);
					});
				});
			});
		},

		codemirror: function() {
			$(function() {
				var editorsSmarty = [],
					editorsSass = [],
					editorsHtml = [],
					editorsCSS = [],
					editorsJS = [],
					editorsSQL = [];


				function wc_hex_is_light(color) {
					const hex = color.replace('#', '');
					const c_r = parseInt(hex.substr(0, 2), 16);
					const c_g = parseInt(hex.substr(2, 2), 16);
					const c_b = parseInt(hex.substr(4, 2), 16);
					const brightness = ((c_r * 299) + (c_g * 587) + (c_b * 114)) / 1000;
					return brightness > 155;
				}

				$(document).ready(function () {
					var idListSmarty = $('.css-codemirror.smarty'),
						idListHTML = $('.css-codemirror.html'),
						idListSASS = $('.css-codemirror.sass'),
						idListSQL = $('.css-codemirror.sql'),
						idListCSS = $('.css-codemirror.css'),
						idListJavaScript = $('.css-codemirror.JavaScript'),
						codemirrorTheme = wc_hex_is_light($(':root').css('--body-bg')) ? 'default' : 'ayu-dark';
					idListHTML.each(function (idx, elem) {
						if (elem.id && elem.id.length > 0 && $("#" +elem.id ).hasClass("initialized") == false) {
							editorsHtml[elem.id] = CodeMirror.fromTextArea(document.getElementById(elem.id), {
								lineNumbers:    true,
								mode:           'htmlmixed',
								scrollbarStyle: 'simple',
								lineWrapping:   true,
								readOnly:	($("#"+elem.id).hasClass("readonly")) ? true : false,
								extraKeys:      {
									'Ctrl-Space': function (cm) {
										cm.setOption('fullScreen', !cm.getOption('fullScreen'));
									},
									'Esc':        function (cm) {
										if (cm.getOption('fullScreen')) cm.setOption('fullScreen', false);
									}
								},
								theme: codemirrorTheme
							});
							$("#" +elem.id ).addClass("initialized");
							$("#cssExpertModus").on('shown.bs.collapse', function () {
								editorsHtml[elem.id].refresh();
							});
							$('#myModalInfo').on('shown.bs.modal', function (e) {
								editorsHtml[elem.id].refresh();
							});
							$('.tabs-nav a[data-toggle="tab"]').on('shown.bs.tab', function (e) {
								editorsHtml[elem.id].refresh();
							});
						}
					});
					idListJavaScript.each(function (idx, elem) {
						if (elem.id && elem.id.length > 0 && $("#" +elem.id ).hasClass("initialized") == false) {
							editorsJS[elem.id] = CodeMirror.fromTextArea(document.getElementById(elem.id), {
								lineNumbers:    true,
								mode:           'javascript',
								scrollbarStyle: 'simple',
								lineWrapping:   true,
								readOnly:	($("#"+elem.id).hasClass("readonly")) ? true : false,
								extraKeys:      {
									'Ctrl-Space': function (cm) {
										cm.setOption('fullScreen', !cm.getOption('fullScreen'));
									},
									'Esc':        function (cm) {
										if (cm.getOption('fullScreen')) cm.setOption('fullScreen', false);
									}
								},
								theme: codemirrorTheme
							});
							$("#" +elem.id ).addClass("initialized");
							$("#cssExpertModus").on('shown.bs.collapse', function () {
								editorsJS[elem.id].refresh();
							});
							$('#myModalInfo').on('shown.bs.modal', function (e) {
								editorsJS[elem.id].refresh();
							});
							$('.tabs-nav a[data-toggle="tab"]').on('shown.bs.tab', function (e) {
								editorsJS[elem.id].refresh();
							});
						}
					});
					idListCSS.each(function (idx, elem) {
						if (elem.id && elem.id.length > 0 && $("#" +elem.id ).hasClass("initialized") == false) {
							editorsCSS[elem.id] = CodeMirror.fromTextArea(document.getElementById(elem.id), {
								lineNumbers:    true,
								mode:           'css',
								scrollbarStyle: 'simple',
								lineWrapping:   true,
								readOnly:	($("#"+elem.id).hasClass("readonly")) ? true : false,
								extraKeys:      {
									'Ctrl-Space': function (cm) {
										cm.setOption('fullScreen', !cm.getOption('fullScreen'));
									},
									'Esc':        function (cm) {
										if (cm.getOption('fullScreen')) cm.setOption('fullScreen', false);
									}
								},
								theme: codemirrorTheme
							});
							$("#" +elem.id ).addClass("initialized");
							$("#cssExpertModus").on('shown.bs.collapse', function () {
								editorsCSS[elem.id].refresh();
							});
							$('#myModalInfo').on('shown.bs.modal', function (e) {
								editorsCSS[elem.id].refresh();
							});
							$('.tabs-nav a[data-toggle="tab"]').on('shown.bs.tab', function (e) {
								editorsCSS[elem.id].refresh();
							});
						}
					});
					idListSASS.each(function (idx, elem) {
						if (elem.id && elem.id.length > 0 && $("#" +elem.id ).hasClass("initialized") == false) {
							editorsSass[elem.id] = CodeMirror.fromTextArea(document.getElementById(elem.id), {
								lineNumbers:    true,
								mode:           'sass',
								scrollbarStyle: 'simple',
								lineWrapping:   true,
								readOnly:	($("#"+elem.id).hasClass("readonly")) ? true : false,
								extraKeys:      {
									'Ctrl-Space': function (cm) {
										cm.setOption('fullScreen', !cm.getOption('fullScreen'));
									},
									'Esc':        function (cm) {
										if (cm.getOption('fullScreen')) cm.setOption('fullScreen', false);
									}
								},
								theme: codemirrorTheme
							});
							$("#" +elem.id ).addClass("initialized");
							$("#cssExpertModus").on('shown.bs.collapse', function () {
								editorsSass[elem.id].refresh();
							});
							$('#myModalInfo').on('shown.bs.modal', function (e) {
								editorsSass[elem.id].refresh();
							});
							$('.tabs-nav a[data-toggle="tab"]').on('shown.bs.tab', function (e) {
								editorsSass[elem.id].refresh();
							});
						}
					});
					idListSmarty.each(function (idx, elem) {
						if (elem.id && elem.id.length > 0 && $("#" +elem.id ).hasClass("initialized") == false) {
							editorsSmarty[elem.id] = CodeMirror.fromTextArea(document.getElementById(elem.id), {
								lineNumbers:    true,
								lineWrapping:   true,
								mode:           'smartymixed',
								scrollbarStyle: 'simple',
								readOnly:	($("#"+elem.id).hasClass("readonly")) ? true : false,
								extraKeys:      {
									'Ctrl-Space': function (cm) {
										cm.setOption('fullScreen', !cm.getOption('fullScreen'));
									},
									'Esc':        function (cm) {
										if (cm.getOption('fullScreen')) cm.setOption('fullScreen', false);
									}
								},
								theme: codemirrorTheme
							});
							$("#" +elem.id ).addClass("initialized");
							$("#cssExpertModus").on('shown.bs.collapse', function () {
								editorsSmarty[elem.id].refresh();
							});
							$('#myModalInfo').on('shown.bs.modal', function (e) {
								editorsSmarty[elem.id].refresh();
							});
							$('.tabs-nav a[data-toggle="tab"]').on('shown.bs.tab', function (e) {
								editorsSmarty[elem.id].refresh();
							});
						}
					});
					idListSQL.each(function (idx, elem) {
						if (elem.id && elem.id.length > 0 && $("#" +elem.id ).hasClass("initialized") == false) {
							var hint = $('#' + elem.id).data('hint');
							editorsSQL[elem.id] = CodeMirror.fromTextArea(document.getElementById(elem.id), {
								mode: 'text/x-mysql',
								scrollbarStyle: 'simple',
								lineWrapping:   true,
								smartIndent: true,
								lineNumbers: true,
								matchBrackets : true,
								autofocus: true,
								readOnly:	($("#"+elem.id).hasClass("readonly")) ? true : false,
								extraKeys: {"Ctrl-Space": "autocomplete"},
								hintOptions: hint,
								theme: codemirrorTheme
							});
						}
						$("#" +elem.id ).addClass("initialized");
						$("#cssExpertModus").on('shown.bs.collapse', function () {
							editorsSQL[elem.id].refresh();
						});
						$('#myModalInfo').on('shown.bs.modal', function (e) {
							editorsSQL[elem.id].refresh();
						});
						$('.tabs-nav a[data-toggle="tab"]').on('shown.bs.tab', function (e) {
							editorsSQL[elem.id].refresh();
						});
					});
				});

				// Handler for .ready() called.
			});

		},




		post_data: function (url, data) {
			var that = this;
			that.debug(url);
			jQuery.ajax({
				type: 'POST',
				url: url,
				data: data,
				showNoSuggestionNotice: true,
				dataType: 'json',
				beforeSend: function(jqXHR) {
					jqXHR.overrideMimeType('text/json;charset=iso-8859-1');
				},
				success: function(data,responseText,jqXHR) {
					if( jqXHR.status == 204 ) {
						that.debug('sorry, here is nothing to do for me. (Token problem?)')
						return false;
					}
					if( jqXHR.status == 404 ) {
						that.debug('sorry, here is nothing to show you.')
						return false;
					}
					if( jqXHR.status == 403 ) {
						that.debug('sorry, this is no place for you.')
						return false;
					}
					//debug(data);
					if( data.script )	{
						eval(data.script);
					}
				},
				error: function() {
					that.debug('something is wrong....anything buggy?');
				},
				complete: function() {
				}
			});
		},
	};

	$.cssUploadFormularAdmin = function(options) {
		console.log("starte cssUploadFormular");
		$.cssUploadFormularAdmin = new cssUploadFormularAdminClass(options);
		$.cssUploadFormularAdmin.register();
		return $.cssUploadFormularAdmin;
	};
	$.cssUploadFormularAdmin();
})(jQuery);



